@@ -34,3 +34,12 @@ class SearchMirror4Test(BaseTest):
     fixtureDB = True
     outputMatchPrepare = lambda _, s: "\n".join(sorted(s.split("\n")))
     runCmd = "aptly mirror search -with-deps wheezy-main 'Name (nginx)'"
+
+
+class SearchMirror5Test(BaseTest):
+    """
+    search mirror: regular search
+    """
+    fixtureDB = True
+    outputMatchPrepare = lambda _, s: "\n".join(sorted(s.split("\n")))
+    runCmd = "aptly mirror search -format='{{.Package}}#{{.Version}}' wheezy-main '$$Architecture (i386), Name (% *-dev)'"
